import { UserActionType, GetAllUsersActionSuccess, GetAllUsersActionError, UserAction, SearchUserActionSuccess, SearchUserActionError } from './users.actions';
import { map, mergeMap, Observable, catchError, of } from 'rxjs';
import { UsersService } from './../../shared/services/users.service';
import { Injectable } from "@angular/core";
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { User } from 'src/app/shared/models/user.model';

@Injectable()
export class UserEffects{

  constructor(private usersService: UsersService, private effectActions: Actions){}

  //create getAllUsers effects

  getAllUsersEffect: Observable<UserAction> = createEffect(

    () => this.effectActions.pipe(
      ofType(UserActionType.GET_ALL_USERS),
      mergeMap(
        (userAction: UserAction) =>{

          return this.usersService.getAllUsers().pipe(
              map((users: User[])=> new GetAllUsersActionSuccess(users)),
              catchError((err) => of(new GetAllUsersActionError(err.message)))
            );
        }
      )

    )
  );

   //create searchUser effects

   searchUserEffect: Observable<UserAction> = createEffect(

    () => this.effectActions.pipe(
      ofType(UserActionType.SEARCH_A_USER),
      mergeMap(
        (userAction: UserAction) =>{

          return this.usersService.searchUser(userAction.payload).pipe(
              map((users: User[])=> new SearchUserActionSuccess(users)),
              catchError((err) => of(new SearchUserActionError(err.message)))
            );
        }
      )

    )
  );
}
