import { Action } from '@ngrx/store';
import { User } from 'src/app/shared/models/user.model';
import { UserAction, UserActionType } from './users.actions';

export enum UserStateEnum {
  LOADING = '[User] LOADING',
  LOADED = '[User] LOADED',
  ERROR = '[User] ERROR',
  INIT_STATE = '[User] INITIAL STATE',
}

//create a state
export interface UserState {
  usersList: User[];
  serverErrorMsg: string;
  dataState: UserStateEnum;
}

//create reducer, a reducer need a initial state and an action
const initUserState: UserState = {
  usersList: [],
  serverErrorMsg: '',
  dataState: UserStateEnum.INIT_STATE,
};

export function userReducer(
  state: UserState = initUserState,
  action: Action
): UserState {
  switch (action.type) {
    case UserActionType.GET_ALL_USERS:
      return { ...state, dataState: UserStateEnum.LOADING };

    case UserActionType.GET_ALL_USERS_SUCCESS:
      return {
        ...state,
        dataState: UserStateEnum.LOADED,
        usersList: (<UserAction>action).payload,
      };

    case UserActionType.GET_ALL_USERS_ERROR:
      return {
        ...state,
        dataState: UserStateEnum.ERROR,
        serverErrorMsg: (<UserAction>action).payload,
      };

    case UserActionType.SEARCH_A_USER:
      return { ...state, dataState: UserStateEnum.LOADING };

    case UserActionType.SEARCH_A_USER_SUCCESS:
      return {
        ...state,
        dataState: UserStateEnum.LOADED,
        usersList: (<UserAction>action).payload,
      };

    case UserActionType.SEARCH_A_USER_ERROR:
      return {
        ...state,
        dataState: UserStateEnum.ERROR,
        serverErrorMsg: (<UserAction>action).payload,
      };

    default:
      return { ...state };
  }
}
